;==========================================================
; ASELECT.LSP Copyright 1995 by Looking Glass Microproducts
;==========================================================
; Select Blocks by their Attribute Values
;==========================================================
(defun C:ASELECT (/ ERROR XGETSTRING ITEM BUMP BUMPNO 
                    QUALIFY LAYER BLOCK TAG VALUE FILTER 
                    SS1 N HIGHLIGHT)
   ;=======================================================
   ; Error Handler
   ;=======================================================
   (defun ERROR (S)
      (if (not
             (member
                S
                '("Function cancelled" "console break")
             )
          )
         (princ (strcat "\n" S))
      )
      (setq *error* OLD_ERROR)
      (princ)
   )
   ;=======================================================
   ; Get String with default
   ;=======================================================
   (defun XGETSTRING (CR PRMPT DEFAULT / TEMP)
      (cond
         ((=
             ""
             (setq
                TEMP (getstring
                        CR
                        (if (/= "" DEFAULT)
                           (strcat
                              PRMPT
                              " <"
                              DEFAULT
                              ">: "
                           )
                           (strcat PRMPT ": ")
                        )
                     )
             )
          )
            DEFAULT
         )
         ((= "." TEMP) "")
         (t TEMP)
      )
   )
   ;=======================================================
   ; Item from association list
   ;=======================================================
   (defun ITEM (A B) (cdr (assoc A B)))
   ;=======================================================
   ; Entertainment
   ;=======================================================
   (setq BUMPNO 0)
   (defun BUMP ()
      (setq BUMPNO (logand (1+ BUMPNO) 3))
      (princ
         (strcat
            "\rProcessing ["
            (nth BUMPNO '("|" "/" "-" "\\"))
            "]... "
         )
      )
   )
   ;=======================================================
   ; Remove any blocks from ss1 that don't have an
   ; attribute that matches tag and value
   ;======================================================= 
   (defun QUALIFY (SS1 TAG VALUE / N ANAME BNAME MATCH)
      (princ "\n")
      (setq N (sslength SS1))
      (while (> N 0)
         (BUMP)
         (setq
            N     (1- N)
            BNAME (ssname SS1 N)
            ANAME BNAME
            MATCH nil
         )
         (while (and
                   (=
                      "ATTRIB"
                      (ITEM
                         0
                         (setq
                            ANAME (entnext ANAME)
                            AENT  (entget ANAME)
                         )
                      )
                   )
                   (not MATCH)
                )
            ;--------------------------------------------
            ; Is this the attribute with the tag we want?
            ;--------------------------------------------
            (setq
               MATCH (and
                        (wcmatch 
                           (strcase (ITEM 2 AENT)) 
                           TAG
                        )
                        (wcmatch
                           (strcase (ITEM 1 AENT))
                           VALUE
                        )
                     )
            )
         )
         (if (not MATCH) (ssdel BNAME SS1))
      )
   )
   ;=======================================================
   ; Main Routine
   ;=======================================================
   (setq
      LAYER  (strcase
                (XGETSTRING
                   nil
                   "\nLayer name specification"
                   "*"
                )
             )
      BLOCK  (strcase
                (XGETSTRING
                   nil
                   "\nBlock name specification"
                   "*"
                )
             )
      TAG    (strcase
                (XGETSTRING
                   nil
                   "\nAttribute tag specification"
                   "*"
                )
             )
      VALUE  (strcase
                (XGETSTRING
                   t
                   "\nAttribute value specification"
                   "*"
                )
             )
      FILTER (list
                (cons 0 "INSERT")
                (cons 2 BLOCK)
                (cons 8 LAYER)
                (cons 66 1)
             )
      SS1    (ssget "x" FILTER)
   )
   (cond
      ((null SS1))
      ((progn 
          (QUALIFY SS1 TAG VALUE) 
          (zerop (sslength SS1))
       )
         (setq SS1 nil)
      )
      ((zerop (getvar "cmdactive"))
         (setvar "cmdecho" 0)
         (setq
            HIGHLIGHT (getvar "highlight")
         )
         (setvar "highlight" 0)
         (command ".select" SS1 "")
         (setvar "highlight" HIGHLIGHT)
      )
   )
   ;-------------------------------------------
   ; Exit quietly if called from command prompt
   ;-------------------------------------------
   (if (zerop (getvar "cmdactive"))
      (progn
         (setq N (if SS1 (sslength SS1) 0))
         (princ (strcat (itoa N) " found"))
         (princ)
      )
      SS1
   )
)
(princ
   (strcat
      "  ASELECT.LSP v1.0 (Copyright 1995 by "
      "Looking Glass Microproducts) loaded."
   )
)
(princ)

